/*
 * i2cSlave.c
 *
 *  Created on: 15.11.2010
 *      Author: bernd
 */
#include "i2cSlave.h"

// I2C state and address variables
static volatile eI2cStateType I2cState;
// send/transmit buffer (outgoing data)
static byte I2cSendData[I2C_SEND_DATA_BUFFER_SIZE];
static byte I2cSendDataIndex;
static byte I2cSendDataLength;
// receive buffer (incoming data)
static byte I2cReceiveData[I2C_RECEIVE_DATA_BUFFER_SIZE];
static byte I2cReceiveDataIndex;
static byte I2cReceiveDataLength;


// function pointer to i2c receive routine
//! I2cSlaveReceive is called when this processor
// is addressed as a slave for writing
static void (*i2cSlaveReceive)(byte receiveDataLength, byte* recieveData);
//! I2cSlaveTransmit is called when this processor
// is addressed as a slave for reading
static byte (*i2cSlaveTransmit)(byte transmitDataLengthMax, byte* transmitData);

void i2cInit(uint8_t slave_addr) {
	TWAR = slave_addr & 0xFE; // set slave address, ignore general call 0x00
	TWDR = 0x00; // default initial value
	// Start Slave Listening: Clear TWINT Flag, Enable ACK, Enable TWI, TWI Interrupt Enable
	TWCR = (1 << TWINT) | (1 << TWEA) | (1 << TWEN) | (1 << TWIE);
}

void i2cSetSlaveReceiveHandler(void(*i2cSlaveRx_func)(byte receiveDataLength,
		byte* recieveData)) {
	i2cSlaveReceive = i2cSlaveRx_func;
}

void i2cSetSlaveTransmitHandler(byte(*i2cSlaveTx_func)(
		byte transmitDataLengthMax, byte* transmitData)) {
	i2cSlaveTransmit = i2cSlaveTx_func;
}

ISR(TWI_vect)
{
	static byte i2c_state;
	byte twi_status;
	cli();
	// Disable Global Interrupt
   //uart_puts("ISR ");
	// Get TWI Status Register, mask the prescaler bits (TWPS1,TWPS0)
	twi_status = TWSR & 0xF8;

	switch (twi_status) {
	case TW_SR_SLA_ACK: // 0x60: SLA+W received, ACK returned
		i2c_state = 0; // Start I2C State for Register Address required
		I2cReceiveDataIndex = I2cSendDataIndex = 0;
		TWCR |= (1 << TWINT); // Clear TWINT Flag
		i2c_state = I2C_SLAVE_RX;
		break;

	case TW_SR_DATA_ACK: // 0x80: data received, ACK returned
		I2cReceiveData[I2cReceiveDataIndex++] = TWDR;
		I2cReceiveDataLength++;
		TWCR |= (1 << TWINT); // Clear TWINT Flag
		I2cState = I2C_SLAVE_RX;
		break;

	case TW_SR_STOP: // 0xA0: stop or repeated start condition received while selected
		TWCR |= (1 << TWINT); // Clear TWINT Flag

		// if stop condition received, call receive procedure
		i2cSlaveReceive(I2cReceiveDataIndex, I2cReceiveData);

		I2cState = I2C_IDLE;
		break;

	case TW_ST_SLA_ACK: // 0xA8: SLA+R received, ACK returned
	case TW_ST_DATA_ACK: // 0xB8: data transmitted, ACK received
		TWCR |= (1 << TWINT); // Clear TWINT Flag
		I2cState = I2C_SLAVE_TX;
		break;

	case TW_ST_DATA_NACK: // 0xC0: data transmitted, NACK received
	case TW_ST_LAST_DATA: // 0xC8: last data byte transmitted, ACK received
	case TW_BUS_ERROR: // 0x00: illegal start or stop condition
	default:
		I2cReceiveDataIndex = I2cSendDataIndex = 0;
		TWCR |= (1 << TWINT); // Clear TWINT Flag
		I2cState = I2C_ERROR_NODEV;
	}

	// Enable Global Interrupt
	sei();
}

