/*
 * main.c
 *
 *
 * Kommando-Struktur
 *
 * Receive-Data
 *
 * <SlaveID><CMD><DataByte1..n>
 *
 * SlaveID:		0x4E
 * CMD:			01-09 = Servokommandos
 * 				10-19 = Patternkommandos
 * 				20-29 = ADC-Kommandos
 * 				30-39 = Register EEPROM
 *
 * 01-09 Servokommandos
 * 01	Servo absolut positionieren
 * 		Daten:	<ServoID (0-7)> <Position 0-255>
 *
 * 02 	Servo Winkelposition
 * 		Daten:  <ServoID (0-7)> <Winkel 0-180>
 *
 *
 * 10-19 Patternkommandos
 * 10	Pattern fr definierte Servos ausfhren
 * 		Daten: <PatternID 0..255>
 *
 *
 *
 *
 *
 *
 *
 *
 *
 *  Created on: 15.11.2010
 *      Author: bernd
 */

#include <avr/io.h>
#include <util/delay.h>
#include "general.h"
#include "project.h"
#include "uart.h"
#include "i2cSlave.h"
#include "servo2.h"

#define UART0_BAUD 38400
byte localbuffer[] = "1234567890";
byte localBufferLength = 0x00;
byte status;
//--------- defines -----------------
#define CMD_SERVOPOS 	0x01
#define CMD_SERVODEG 	0x02
#define CMD_USEPATT		0x10

// --------- prototypes --------------------
void init();
void i2cSlaveReceiveService(byte receiveDataLength, byte* receiveData);
byte i2cSlaveTransmitService(byte transmitDataLengthMax, byte* transmitData);
byte checkCommand();



// --------- functions ---------------------
void i2cSlaveReceiveService(byte receiveDataLength, byte* receiveData) {
#ifndef SIMULATE
	uart_puts("received data...");
#endif
	localBufferLength = receiveDataLength;
	for (int x = 0; x < receiveDataLength; x++) {
		localbuffer[x] = *receiveData++;
	}
	if (checkCommand() > 0) {
#ifndef SIMULATE
		uart_puts("command error");
#endif
	}
	for (int x = 0; x < I2C_RECEIVE_DATA_BUFFER_SIZE; x++) {
		localbuffer[x] = 0;
	}

	// reset buffer

}

byte i2cSlaveTransmitService(byte transmitDataLengthMax, byte* transmitData) {

	return 0;
}

byte checkCommand() {
	status = 0;
	switch (localbuffer[0]) {
	case CMD_SERVOPOS:
		if (localBufferLength >= 4) {
			setServoPos(localbuffer[1], localbuffer[2]);
		}
		else {
			status = 1;
		}
		break;
	case CMD_SERVODEG:
		if (localBufferLength >= 4) {
			setServoAngle(localbuffer[1], localbuffer[2]);
		}
		else {
			status = 1;
		}

		break;
	case CMD_USEPATT:
		status = 1;
		break;

	}
	return status;
}


/*
 * initialize slave
 */
void init() {
	uart_init(UART_BAUD_SELECT(UART0_BAUD,F_CPU));
	i2cSetSlaveReceiveHandler(i2cSlaveReceiveService);
	i2cInit(SERVOCONTROL_ADDRESS);
	init_servo();
	sei();
#ifndef SIMULATE
	uart_puts("ATMega Servo-Controller V0-1\n");
#endif
	for (int x = 0; x < MAX_SERVOS;x++) {
		if (x % 2 == 0)
			setServoPos(x,255);
		else
			setServoPos(x,10);
	}
}
/***********************************************************************************************
 *
 *
 *
 ***********************************************************************************************/
int main(void) {
	init();
	for (;;) {
		FlashLed50(&LED1_PORT, LED1_BIT, 1);
#ifndef SIMULATE
		uart_putc('.');
		_delay_ms(100);
#else
		status = 0;
#endif
	}

}

